import { Box } from '../../../classes/display/box.js';
import { Scene } from '../../../classes/display/scene.js';
import { Expert } from '../../../classes/actors/expert.js';
import { PostContent } from '../../../classes/util/post-content.js';
import { delay } from '../../../util.js';
import params from '../../../params.js';
import { DAO } from '../../../classes/actors/dao.js';

export class ForumTest {
  constructor(options) {
    this.scene = null;
    this.dao = null;
    this.experts = null;
    this.posts = null;
    this.options = {
      defaultDelayMs: 1,
      poolDurationMs: 50,
      ...options,
    };
  }

  async addPost(author, fee, citations = []) {
    const postIndex = this.posts.length;
    const title = `posts[${postIndex}]`;
    await this.scene.sequence.startSection();

    const postContent = new PostContent({}).setTitle(title);
    for (const { postId, weight } of citations) {
      postContent.addCitation(postId, weight);
    }

    const { pool, postId } = await author.submitPostWithFee(
      postContent,
      {
        fee,
        duration: this.options.poolDurationMs,
        tokenLossRatio: 1,
      },
    );
    this.posts.push(postId);
    await delay(this.options.poolDurationMs);
    await pool.evaluateWinningConditions();
    await this.scene.sequence.endSection();
    await delay(this.options.defaultDelayMs);
    return postId;
  }

  async newExpert() {
    const index = this.experts.length;
    const name = `Expert${index + 1}`;
    const expert = await new Expert(this.dao, name, this.scene).initialize();
    this.experts.push(expert);
    // await expert.addComputedValue('rep', () => this.dao.reputation.valueOwnedBy(expert.reputationPublicKey));
    return expert;
  }

  async setup() {
    const rootElement = document.getElementById('scene');
    const rootBox = new Box('rootBox', rootElement).flex();

    const scene = this.scene = new Scene('Forum test', rootBox);
    scene.withSequenceDiagram();
    scene.withFlowchart();
    scene.withTable();

    scene.addDisplayValue('c3. stakeForAuthor').set(params.stakeForAuthor);
    scene.addDisplayValue('q2. revaluationLimit').set(params.revaluationLimit);
    scene
      .addDisplayValue('q3. referenceChainLimit')
      .set(params.referenceChainLimit);
    scene.addDisplayValue('q4. leachingValue').set(params.leachingValue);
    scene.addDisplayValue('&nbsp;');

    this.dao = new DAO('DAO', scene);
    this.forum = this.dao.forum;
    this.experts = [];
    this.posts = [];

    await this.newExpert();
    // await newExpert();
    // await newExpert();

    await this.dao.addComputedValue('total value', () => this.dao.reputation.getTotal());
    // await this.dao.addComputedValue('total reputation', () => this.dao.forum.getTotalValue());
    this.dao.computeValues();
  }
}
